import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;

import ru.amse.fedorov.plainsvg.gui.ComponentSelectedList;
import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;

/**
 * Main UI class of "Plain SVG" editor.
 * 
 * @author Alexey Fedorov
 */
public class MainFrame extends JFrame {

	private static final long serialVersionUID = 1;

	/*
	 * Size of the bound field of the component
	 */
	public static final int FIELD = 20;

	/*
	 * Frame sizes
	 */
	private static final int DEFAULT_WIDTH = 1020;

	private static final int DEFAULT_HEIGHT = 730;

	private final LookingPanel backgroundPanel;

	/*
	 * Main graphics component.
	 */
	private final GraphicsComponent component = new GraphicsComponent();

	// Structure of menu description:

	/*
	 * Menubar
	 */
	private final SVGMenuBar menubar = new SVGMenuBar(component, this);

	/*
	 * The list contained all selected elements.
	 */
	private final ComponentSelectedList selectedList = component
			.getSelectedList();

	private final SVGToolBar toolbar = new SVGToolBar(component);

	private final PropertiesToolbar propertiesToolbar = new PropertiesToolbar(
			component);

	private final JPanel leftPanel = new JPanel();

	private final JToolBar mainToolBar = new MainToolBar(component);

	/**
	 * Creates Frame: initializes menu, all main elements, sets position and
	 * sise and adds main elements.
	 */
	private MainFrame() {
		super("Plain SVG");
		setIconImage(new ImageIcon(ClassLoader.getSystemResource("svg.gif"))
				.getImage());
		backgroundPanel = new LookingPanel(component);
		setPositionAndSize();
		addMainElements();
		component.getElementList().addModelChangedListener(
				component.getSuperListener());
		selectedList.addSelectionChangedListener(propertiesToolbar);
		setMinimumSize(new Dimension(700, 400));
		component.setVisible(false);

		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {

			@Override
			public void windowClosing(WindowEvent e) {
				menubar.getCloseAction().actionPerformed(
						new ActionEvent(e.getSource(), e.getID(), ""));
			}

		});
	}

	/*
	 * Sets start position and frame size.
	 */
	private void setPositionAndSize() {
		setSize(DEFAULT_WIDTH, DEFAULT_HEIGHT);
		this.setLocation(0, 0);
	}

	/*
	 * Adds all main elements.
	 */
	private void addMainElements() {
		final int field = FIELD;
		setJMenuBar(menubar);
		leftPanel.add(toolbar);
		getContentPane().add(leftPanel, BorderLayout.WEST);

		backgroundPanel.setLayout(null);
		backgroundPanel.add(component);
		component.addComponentListener(backgroundPanel);
		component.setLocation(field, field);
		getContentPane().add(propertiesToolbar, BorderLayout.SOUTH);
		getContentPane().add(mainToolBar, BorderLayout.NORTH);
		getContentPane().add(new JScrollPane(backgroundPanel),
				BorderLayout.CENTER);
	}

	/**
	 * MAIN method.
	 * 
	 * @param args
	 *            no parameters
	 */
	public static void main(String[] args) {
		new MainFrame().setVisible(true);
	}

	private class LookingPanel extends JPanel implements ComponentListener {

		/**
		 * serialVersionUID
		 */
		private static final long serialVersionUID = 1L;

		private JComponent comp;

		public LookingPanel(JComponent comp) {
			super();
			this.comp = comp;
			int w = comp.getWidth();
			int h = comp.getHeight();
			Dimension boundedDim = new Dimension(w + 2 * FIELD, h + 2 * FIELD);
			setSize(boundedDim);
			setPreferredSize(boundedDim);
		}

		public void componentHidden(ComponentEvent e) {
			// Nothing !
		}

		public void componentMoved(ComponentEvent e) {
			// Nothing !
		}

		public void componentResized(ComponentEvent e) {
			int w = comp.getWidth();
			int h = comp.getHeight();
			Dimension boundedDim = new Dimension(w + 2 * FIELD, h + 2 * FIELD);
			setSize(boundedDim);
			setPreferredSize(boundedDim);
		}

		public void componentShown(ComponentEvent e) {
			// Nothing !
		}

	}

}
