import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.InputEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.gui.actions.ChangeToolAction;
import ru.amse.fedorov.plainsvg.gui.actions.edit.DeleteAction;
import ru.amse.fedorov.plainsvg.gui.actions.edit.SelectAllAction;
import ru.amse.fedorov.plainsvg.gui.actions.file.AbstractSaveAction;
import ru.amse.fedorov.plainsvg.tools.ToolStatusListener;

/**
 * The menubar of the editor.
 */
/* package */class SVGMenuBar extends JMenuBar {

	private static final long serialVersionUID = 1L;

	private static final String VERSION = "0.3.2";

	private static final String DATE = "2007, May 7";

	private static final String ABOUT_MESSAGE = "\n"
			+ "Plain SVG - simple SVG-editor\n\n"
			+ "AUTHOR: Alexey Fyodorov\n\n"
			+ "COMPANY: Academy of Modern Software Engineering\n\n"
			+ "COMPANY SITE: http://www.amse.ru\n\n" + "VERSION: " + VERSION
			+ "   (" + DATE + ")\n\n";

	/*
	 * The menu for file operations.
	 */
	private final JMenu fileMenu = new JMenu("File");

	/*
	 * The menu for edit operations.
	 */
	private final JListeningMenu editMenu = new JListeningMenu("Edit");

	/*
	 * The menu for changing order of svg-elements.
	 */
	private final JListeningMenu moveMenu = new JListeningMenu("Move");

	/*
	 * The menu for changing image.
	 */
	private final JListeningMenu imageMenu = new JListeningMenu("Image");

	/*
	 * The menu for zooming.
	 */
	private final JListeningMenu zoomMenu = new JListeningMenu("Zoom");

	/*
	 * The menu for changing tool.
	 */
	private final JListeningMenu toolsMenu = new JListeningMenu("Tools");

	/*
	 * The menu for help.
	 */
	private final JListeningMenu helpMenu = new JListeningMenu("Help");

	/*
	 * Main graphics component.
	 */
	private final GraphicsComponent graphicsComponent;

	/*
	 * Group of tool-buttons.
	 */
	private final ButtonGroup group = new ButtonGroup();
	
	private final Action closeAction;

	/**
	 * Initialises all menuBar elements
	 * 
	 * @param component
	 *            Main graphics component
	 * 
	 * @see ru.amse.fedorov.plainsvg.gui.GraphicsComponent
	 */
	/* package */SVGMenuBar(GraphicsComponent component, JFrame frame) {
		graphicsComponent = component;
		fileMenu.add(graphicsComponent.getNewAction());
		fileMenu.add(graphicsComponent.getOpenAction());
		fileMenu.addSeparator();
		fileMenu.add(graphicsComponent.getSaveAction());
		fileMenu.add(graphicsComponent.getSaveAsAction());
		fileMenu.addSeparator();
		closeAction = new AbstractSaveAction(graphicsComponent, "Exit") {

			{
				putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(
						new Integer('X'), InputEvent.CTRL_MASK));
			}

			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				if (proposeSave()) {
					System.exit(0);
				}
			}

		};
		fileMenu.add(closeAction);

		editMenu.add(new DeleteAction(graphicsComponent));
		editMenu.addSeparator();
		editMenu.add(new SelectAllAction(graphicsComponent));

		JCheckBoxMenuItem selectItem = new ToolMenuItem(graphicsComponent
				.getSelectAction());

		JCheckBoxMenuItem addRectItem = new ToolMenuItem(graphicsComponent
				.getAddRectAction());
		JCheckBoxMenuItem addEllipseItem = new ToolMenuItem(graphicsComponent
				.getAddEllipseAction());
		JCheckBoxMenuItem addLineItem = new ToolMenuItem(graphicsComponent
				.getAddLineAction());
		JCheckBoxMenuItem addPolygonItem = new ToolMenuItem(graphicsComponent
				.getAddPolygonAction());
		toolsMenu.addSeparator();
		addToolItem(selectItem);
		toolsMenu.addSeparator();
		addToolItem(addRectItem);
		addToolItem(addEllipseItem);
		addToolItem(addLineItem);
		addToolItem(addPolygonItem);

		JMenuItem aboutItem = new JMenuItem("About...");
		aboutItem.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				JOptionPane
						.showMessageDialog(null, ABOUT_MESSAGE,
								"About Plain SVG",
								JOptionPane.INFORMATION_MESSAGE, new ImageIcon(
										ClassLoader
												.getSystemResource("svg.gif")));
			}

		});
		helpMenu.add(aboutItem);

		moveMenu.add(graphicsComponent.getMoveTop());
		moveMenu.add(graphicsComponent.getMoveUp());
		moveMenu.add(graphicsComponent.getMoveDown());
		moveMenu.add(graphicsComponent.getMoveBottom());
		graphicsComponent.addComponentListener(editMenu);
		graphicsComponent.addComponentListener(moveMenu);
		graphicsComponent.addComponentListener(imageMenu);
		graphicsComponent.addComponentListener(zoomMenu);
		graphicsComponent.addComponentListener(toolsMenu);
		imageMenu.add(new AbstractAction("Properties...") {

			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				graphicsComponent.getPropertiesDialog().setVisible(true);
			}

		});

		zoomMenu.add(graphicsComponent.getZoomIn());
		zoomMenu.add(graphicsComponent.getZoomOut());

		add(fileMenu);
		add(editMenu);
		add(moveMenu);
		add(imageMenu);
		add(zoomMenu);
		add(toolsMenu);
		add(helpMenu);
	}

	private void addToolItem(JCheckBoxMenuItem item) {
		group.add(item);
		toolsMenu.add(item);
	}

	private class ToolMenuItem extends JCheckBoxMenuItem implements
			ToolStatusListener {

		/**
		 * serialVersionUID
		 */
		private static final long serialVersionUID = 1L;

		/**
		 * Creates tool menu item.
		 * 
		 * @param a
		 *            base action
		 */
		public ToolMenuItem(ChangeToolAction a) {
			super(a);
			a.getTool().addToolStatusListener(this);
		}

		public void doOnStatusChanged() {
			setSelected(!isSelected());

		}

	}
	
	

	private class JListeningMenu extends JMenu implements ComponentListener {

		private static final long serialVersionUID = 1L;

		public JListeningMenu(String s) {
			super(s);
		}

		public void componentHidden(ComponentEvent e) {
			setEnabled(false);
		}

		public void componentMoved(ComponentEvent e) {
			// Nothing!
		}

		public void componentResized(ComponentEvent e) {
			// Nothing!
		}

		public void componentShown(ComponentEvent e) {
			setEnabled(true);
		}

	}



	/**
	 * @return the closeAction
	 */
	public Action getCloseAction() {
		return closeAction;
	}

}
