package ru.amse.fedorov.plainsvg;

import java.util.ArrayList;
import java.util.Collection;

/**
 * This is ArrayList with changed modifying methods. Each modifying method
 * notifyind listeners.
 * 
 * @author Alexey Fedorov
 */
public abstract class NotifyingArrayList<E> extends ArrayList<E> {

	public NotifyingArrayList(Collection<? extends E> c) {
		super(c);
	}

	public NotifyingArrayList() {
		super();
	}

	/**
	 * This method notifies all subscribed listeners that data model has been
	 * changed.
	 */
	protected abstract void firePropertyChanged();

	@Override
	public boolean add(E e) {
		boolean result = false;
		result = super.add(e);
		firePropertyChanged();
		return result;
	}

	@Override
	public void add(int index, E element) {
		super.add(index, element);
		firePropertyChanged();
	}

	@Override
	public boolean addAll(int index, Collection<? extends E> c) {
		boolean result = super.addAll(index, c);
		firePropertyChanged();
		return result;
	}

	@Override
	public E remove(int index) {
		E e = super.remove(index);
		firePropertyChanged();
		return e;
	}

	@Override
	public boolean addAll(Collection<? extends E> c) {
		boolean result = super.addAll(c);
		firePropertyChanged();
		return result;
	}

	@Override
	public boolean remove(Object o) {
		boolean result = super.remove(o);
		if (result) {
			firePropertyChanged();
		}
		return result;
	}

	@Override
	public boolean removeAll(Collection<?> c) {
		boolean result = super.removeAll(c);
		if (result) {
			firePropertyChanged();
		}
		return result;
	}

	@Override
	public boolean retainAll(Collection<?> c) {
		boolean result = super.retainAll(c);
		if (result) {
			firePropertyChanged();
		}
		return result;
	}

	@Override
	public E set(int index, E element) {
		E result = super.set(index, element);
		firePropertyChanged();
		return result;
	}

	@Override
	public void clear() {
		super.clear();
		firePropertyChanged();
	}

	/**
	 * Adds element to the end of the list.
	 * 
	 * @param element
	 *            added element
	 */
	public void addToEnd(E element) {
		add(size(), element);
	}

	/**
	 * Adds collection to the end of the list.
	 * 
	 * @param c
	 *            added collection
	 */
	public void addAllToEnd(Collection<? extends E> c) {
		addAll(size(), c);
	}

}
