package ru.amse.fedorov.plainsvg.geom;

import java.awt.geom.Point2D;

public class GeomUtils {

	/**
	 * @return distance between points p1 and p2
	 */
	public static double dist(Point2D p1, Point2D p2) {
		return p1.distance(p2);
	}

	/**
	 * Conculates distance between point p0 and line through-passing points p1
	 * and p2.
	 */
	public static double dist(Point2D p0, Point2D p1, Point2D p2) {
		double a = dist(p0, p1);
		double b = dist(p0, p2);
		double c = dist(p1, p2);
		double p = (a + b + c) / 2;
		double s = Math.sqrt(p * (p - a) * (p - b) * (p - c));
		return 2 * s / c;
	}

	/**
	 * Returns the angle between p1, center and p2.
	 * 
	 * @param center
	 *            The Vertex of angle
	 * @return The angle between two lines.
	 */
	public static double angle(Point2D p1, Point2D center, Point2D p2) {
		Point2D vector1 = new Point2D.Double(p1.getX() - center.getX(), p1
				.getY()
				- center.getY());
		Point2D vector2 = new Point2D.Double(p2.getX() - center.getX(), p2
				.getY()
				- center.getY());
		double sUp2 = (vector1.getX() * vector2.getY() - vector2.getX()
				* vector1.getY());
		double sin = Math.min(sUp2 / (dist(p1, center) * dist(p2, center)), 1);
		return Math.asin(sin);
	}
}
