package ru.amse.fedorov.plainsvg.gui.actions;

import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.InputEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.tools.Tool;

/**
 * Changes tool.
 */
public class ChangeToolAction extends AbstractAction implements
		ComponentListener {

	/**
	 * serialVersionUID
	 */
	private static final long serialVersionUID = 1L;

	/*
	 * Current tool
	 */
	private final Tool tool;

	private final GraphicsComponent component;

	/**
	 * Creates tool action with given parameters
	 * 
	 * @param tool
	 *            corresponding tool
	 * @param name
	 *            name of action (button title)
	 * @param c
	 *            accelerator key for the action
	 */
	public ChangeToolAction(GraphicsComponent component, Tool tool,
			String name, char c) {
		this(component, tool, name);
		putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(new Integer(c),
				InputEvent.CTRL_MASK));
	}

	public ChangeToolAction(GraphicsComponent component, Tool tool, String name) {
		super(name, new ImageIcon(ClassLoader.getSystemResource(name
				.toLowerCase()
				+ ".gif")));
		this.component = component;
		this.tool = tool;
		this.component.addComponentListener(this);
		putValue(Action.SHORT_DESCRIPTION, name + " tool");

	}

	public void actionPerformed(ActionEvent e) {
		component.setCurrentTool(tool);
	}

	/**
	 * @return the tool
	 */
	public Tool getTool() {
		return tool;
	}

	public void componentHidden(ComponentEvent e) {
		setEnabled(false);
	}

	public void componentMoved(ComponentEvent e) {
		// Nothing!
	}

	public void componentResized(ComponentEvent e) {
		// Nothing!
	}

	public void componentShown(ComponentEvent e) {
		setEnabled(true);
	}

}
