package ru.amse.fedorov.plainsvg.gui.actions.file;

import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.gui.SVGFilter;
import ru.amse.fedorov.plainsvg.presentation.elements.ElementPresentation;
import ru.amse.fedorov.plainsvg.presentation.factories.ElementFactory;
import ru.amse.fedorov.plainsvg.presentation.factories.EllipseFactory;
import ru.amse.fedorov.plainsvg.presentation.factories.LineFactory;
import ru.amse.fedorov.plainsvg.presentation.factories.PolygonFactory;
import ru.amse.fedorov.plainsvg.presentation.factories.RectangleFactory;

public class OpenDocumentAction extends AbstractSaveAction {

	private static final long serialVersionUID = 1L;

	private final JFileChooser chooser = new JFileChooser();

	private DocumentBuilder builder = null;

	private final Map<String, ElementFactory<?>> factoryMap = new HashMap<String, ElementFactory<?>>();

	public OpenDocumentAction(GraphicsComponent component) {
		super(component);
		putValue(Action.NAME, "Open...");
		putValue(Action.SMALL_ICON, new ImageIcon(ClassLoader
				.getSystemResource("open.gif")));
		putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(
				new Integer('O'), InputEvent.CTRL_MASK));
		chooser.setFileFilter(new SVGFilter());

		factoryMap.put("rect", new RectangleFactory());
		factoryMap.put("ellipse", new EllipseFactory());
		factoryMap.put("line", new LineFactory());
		factoryMap.put("polygon", new PolygonFactory());
	}

	public void actionPerformed(ActionEvent e) {
		if (proposeSave()) {
			openFile();
		}
	}

	private void openFile() {
		int result = chooser.showOpenDialog(null);
		if (result != JFileChooser.APPROVE_OPTION) {
			return;
		}
		File f = chooser.getSelectedFile();
		getComponent().setFile(f);
		try {
			tryOpenFile(f);
		} catch (ParserConfigurationException e) {
			JOptionPane.showMessageDialog(null,
					"Serious configuration error in " + f.getName());
		} catch (SAXException e) {
			JOptionPane.showMessageDialog(null, "SAX error or warning in "
					+ f.getName());
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, e);
		}

	}

	private void tryOpenFile(File f) throws ParserConfigurationException,
			SAXException, IOException {
		if (builder == null) {
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			builder = factory.newDocumentBuilder();
		}
		Document doc = builder.parse(f);
		Element root = doc.getDocumentElement();
		if (!root.getNodeName().equals("svg")) {
			JOptionPane.showMessageDialog(null, "file " + f.getName()
					+ " is corrupted");
			return;
		}
		NodeList children = root.getChildNodes();
		List<ElementPresentation<?>> list = new ArrayList<ElementPresentation<?>>(
				children.getLength());
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			String name = child.getNodeName();
			if (child instanceof Element) {
				Element element = (Element) child;
				if (factoryMap.containsKey(name)) {
					ElementPresentation<?> elem = factoryMap.get(name)
							.createElement(element, getComponent());
					list.add(elem);
					elem.addMarkers();
				}
			}
		}
		getComponent().reload(list, root);
	}
}
