package ru.amse.fedorov.plainsvg.gui.actions.move;

import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.gui.SelectedList;
import ru.amse.fedorov.plainsvg.gui.SelectionChangedListener;
import ru.amse.fedorov.plainsvg.model.ElementList;
import ru.amse.fedorov.plainsvg.presentation.elements.ElementPresentation;

/**
 * Move action.
 */
public abstract class MoveAction extends AbstractAction implements
		SelectionChangedListener {

	private static final long serialVersionUID = 1L;

	private static final String MOVE = "Move ";

	private ElementList<ElementPresentation<?>> elementList;

	private SelectedList<ElementPresentation<?>> selectedList;

	/**
	 * Creates move action with given name
	 * 
	 * @param name
	 *            name of action
	 */
	public MoveAction(GraphicsComponent component) {
		setEnabled(false);
		elementList = component.getElementList();
		selectedList = component.getSelectedList();
		putValue(Action.NAME, MOVE + getOperationName());
		putValue(Action.SMALL_ICON, new ImageIcon(ClassLoader
				.getSystemResource((MOVE + getOperationName() + ".gif")
						.toLowerCase())));
		putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(new Integer(
				getOperationName().toUpperCase().charAt(0)),
				InputEvent.CTRL_MASK));
		putValue(Action.SHORT_DESCRIPTION, MOVE
				+ getOperationName().toLowerCase() + " all selected elements");
	}

	public void actionPerformed(ActionEvent e) {
		perform(e);
	}

	protected abstract void perform(ActionEvent e);

	/**
	 * @return the elementList
	 */
	public ElementList<ElementPresentation<?>> getElementList() {
		return elementList;
	}

	/**
	 * @return the selectedList
	 */
	public SelectedList<ElementPresentation<?>> getSelectedList() {
		return selectedList;
	}

	protected abstract String getOperationName();

	public void selectionChanged() {
		setEnabled(!selectedList.isEmpty());
	}

}