package ru.amse.fedorov.plainsvg.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import ru.amse.fedorov.plainsvg.NotifyingArrayList;
import ru.amse.fedorov.plainsvg.model.listeners.ModelChangedListener;

/**
 * This class realises notifying ArrayList: its notify all listeners if one of
 * modificating methods has been called.
 * 
 * @author Alexey Fedorov
 * 
 * @param <E>
 *            type of elemenst in the list
 */
public class ElementList<E> extends NotifyingArrayList<E> {

	public ElementList() {
		super();
	}

	public ElementList(Collection<? extends E> c) {
		super(c);
	}

	private static final long serialVersionUID = 1;

	/*
	 * The list of the listeners.
	 */
	private List<ModelChangedListener> listenerList = new ArrayList<ModelChangedListener>();

	/**
	 * Moves element with given index to the top of the list.
	 * 
	 * @param e
	 *            element to move
	 */
	public void moveTop(E e) {
		super.remove(e);
		super.add(size(), e);
	}


	/**
	 * Moves given element index up to 1 position.
	 * 
	 * @param e
	 *            element to move
	 */
	public void moveUp(E e) {
		int index = super.indexOf(e);
		if (index < size() - 1) {
			set(index, get(index + 1));
			set(index + 1, e);
		}
	}

	/**
	 * Moves given element index down to 1 position.
	 * 
	 * @param e
	 *            element to move
	 */
	public void moveDown(E e) {
		int index = super.indexOf(e);
		if (index > 0) {
			set(index, get(index - 1));
			set(index - 1, e);
		}
	}

	/**
	 * Moves given element index to the bottom of the list.
	 * 
	 * @param e
	 *            element to move
	 */
	public void moveBottom(E e) {
		add(0, remove(indexOf(e)));
	}

	@Override
	protected void firePropertyChanged() {
		for (ModelChangedListener listener : listenerList) {
			listener.modelChanged();
		}
	}

	/**
	 * Adds a listener to the list that's notified each time a change to the
	 * data model occurs.
	 * 
	 * @param listener
	 *            the <code>ModelChangedListener</code> to be added
	 */
	public void addModelChangedListener(ModelChangedListener listener) {
		listenerList.add(listener);
	}

	/**
	 * Removes a listener from the list that's notified each time a change to
	 * the data model occurs.
	 * 
	 * @param listener
	 *            the <code>ModelChangedListener</code> to be removed
	 */
	public void removeModelChangedListener(ModelChangedListener listener) {
		listenerList.remove(listener);
	}
}
