package ru.amse.fedorov.plainsvg.model.elements;

import java.awt.Color;
import java.awt.geom.Point2D;

/**
 * Presents base ellipse's fields and methods constrained with colors,
 * coordinates and sizes.
 * 
 * @author Alexey Fedorov
 */
public class SVGEllipse extends SVGRectangularShape {

	/**
	 * Creates ellipse whith givel params
	 * 
	 * @param centerX
	 *            ellipse's center horizontal coordinate
	 * @param centerY
	 *            ellipse's center horizontal coordinate
	 * @param radiusX
	 *            horizontal radius of the ellipse
	 * @param radiusY
	 *            vertical radius of the ellipse
	 * @param fill
	 *            The colour to use for the interior of the ellipse
	 * @param stroke
	 *            The colour for the line actually drawn for the ellipse
	 * @param strokeWidth
	 *            The thickness of the line used to draw the ellipse
	 */
	public SVGEllipse(double cx, double cy, double rx, double ry, Color fill,
			Color stroke, double strokeWidth) {
		super(fill, stroke, strokeWidth);
		initCoordinates(cx, cy, rx, ry);
	}

	protected void initCoordinates(double cx, double cy, double rx, double ry) {
		setX1(cx - rx);
		setY1(cy - ry);
		setX2(cx + rx);
		setY2(cy + ry);
	}

	public SVGEllipse(double cx, double cy, double rx, double ry) {
		super();
		initCoordinates(cx, cy, rx, ry);
	}

	public SVGEllipse(double centerX, double centerY) {
		this(centerX, centerY, 0, 0);
	}

	public SVGEllipse(Point2D point) {
		this(point.getX(), point.getY());
	}

	/**
	 * @return horizontal radius of the ellipse
	 */
	public double getRadiusX() {
		return Math.abs(getX1() - getX2()) / 2;
	}

	/**
	 * @return vertical radius of the ellipse
	 */
	public double getRadiusY() {
		return Math.abs(getY1() - getY2()) / 2;
	}

}
