package ru.amse.fedorov.plainsvg.model.elements;

import java.awt.Color;
import java.awt.geom.Point2D;

/**
 * Presents base rectangle's fields and methods constrained with colors,
 * coordinates and sizes.
 * 
 * @author Alexey Fedorov
 */
public class SVGRectangle extends SVGRectangularShape {

	/**
	 * Negative width and height replaced to positive
	 * 
	 * @param x
	 *            The X-coordinate of the top left corner of the rectangle
	 * @param y
	 *            The Y-coordinate of the top left corner of the rectangle
	 * @param width
	 *            The width of the rectangle
	 * @param height
	 *            The height of the rectangle
	 * @param fill
	 *            The colour to use for the interior of the shape
	 * @param stroke
	 *            The colour for the line actually drawn for the rectangle
	 * @param strokeWidth
	 *            The thickness of the line used to draw the rectangle
	 */
	public SVGRectangle(double x, double y, double width, double height,
			Color fill, Color stroke, double strokeWidth) {
		super(fill, stroke, strokeWidth);
		initCoordinates(x, y, width, height);
	}

	protected void initCoordinates(double x, double y, double w, double h) {
		setX1(x);
		setY1(y);
		setX2(x + w);
		setY2(y + h);
	}

	public SVGRectangle(double x, double y, double width, double height) {
		super();
		initCoordinates(x, y, width, height);
	}

	public SVGRectangle(double x, double y) {
		this(x, y, 0, 0);
	}

	public SVGRectangle(Point2D point) {
		this(point.getX(), point.getY());
	}

	public double getX() {
		return Math.min(getX1(), getX2());
	}

	public double getY() {
		return Math.min(getY1(), getY2());
	}

	public double getWidth() {
		return Math.abs(getX1() - getX2());
	}

	public double getHeight() {
		return Math.abs(getY1() - getY2());
	}	
	
}
