package ru.amse.fedorov.plainsvg.presentation.elements;

import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.geom.Point2D;
import java.awt.geom.RectangularShape;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.model.elements.SVGRectangularShape;
import ru.amse.fedorov.plainsvg.presentation.markers.shape.BLMarker;
import ru.amse.fedorov.plainsvg.presentation.markers.shape.BMarker;
import ru.amse.fedorov.plainsvg.presentation.markers.shape.BRMarker;
import ru.amse.fedorov.plainsvg.presentation.markers.shape.LMarker;
import ru.amse.fedorov.plainsvg.presentation.markers.shape.RMarker;
import ru.amse.fedorov.plainsvg.presentation.markers.shape.RectangularRotateMarker;
import ru.amse.fedorov.plainsvg.presentation.markers.shape.TLMarker;
import ru.amse.fedorov.plainsvg.presentation.markers.shape.TMarker;
import ru.amse.fedorov.plainsvg.presentation.markers.shape.TRMarker;

/**
 * This class presents synthesizing of svg-shape and its' 2d-representing.
 * 
 * @author Alexey Fedorov
 */
public abstract class RectangularShapePresentation<T extends SVGRectangularShape>
		extends ShapePresentation<T> {

	public RectangularShapePresentation(GraphicsComponent comp) {
		super(comp);
	}

	@Override
	public void addAllMarkers() {
		Toolkit tk = Toolkit.getDefaultToolkit();
		Image img = tk.getImage(ClassLoader
				.getSystemResource("rotate cursor.gif"));
		Cursor cursor = tk.createCustomCursor(img, new Point(12, 12), "rotate");
		double y1 = getModelElement().getY1();
		double y2 = getModelElement().getY2();
		getMarkers().add(
				new RectangularRotateMarker<RectangularShapePresentation<?>>(
						this, getComponent(), cursor, y1 < y2));
		getMarkers().add(new TLMarker<RectangularShapePresentation<?>>(this));
		getMarkers().add(new BRMarker<RectangularShapePresentation<?>>(this));
		getMarkers().add(new BLMarker<RectangularShapePresentation<?>>(this));
		getMarkers().add(new TRMarker<RectangularShapePresentation<?>>(this));
		getMarkers().add(new TMarker<RectangularShapePresentation<?>>(this));
		getMarkers().add(new BMarker<RectangularShapePresentation<?>>(this));
		getMarkers().add(new LMarker<RectangularShapePresentation<?>>(this));
		getMarkers().add(new RMarker<RectangularShapePresentation<?>>(this));
	}

	@Override
	public void setDiagonal(Point2D point1, Point2D point2) {
		double x1 = point1.getX();
		double y1 = point1.getY();
		double x2 = point2.getX();
		double y2 = point2.getY();
		getModelElement().setCoordinates(x1, y1, x2, y2);
		double x = Math.min(x1, x2);
		double y = Math.min(y1, y2);
		double w = Math.abs(x1 - x2);
		double h = Math.abs(y1 - y2);
		((RectangularShape) getShape()).setFrame(x, y, w, h);
	}

}
