package ru.amse.fedorov.plainsvg.presentation.elements;

import java.awt.BasicStroke;
import java.awt.Graphics2D;

import org.w3c.dom.Element;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.model.elements.SVGShape;

public abstract class ShapePresentation<T extends SVGShape> extends
		StrokedPresentation<T> {
	
	private final GraphicsComponent component;

	public ShapePresentation(GraphicsComponent comp) {
		super();
		component = comp;
	}

	@Override
	public boolean hasFill() {
		return true;
	}

	/**
	 * Repaints image graphics component if need.
	 */
	@Override
	public void paint(Graphics2D graphics, double scale) {
		if (getModelElement().getFill() != null) {
			graphics.setPaint(getModelElement().getFill());
			graphics.fill(getShape());
		}
		float st = (float) getModelElement().getStrokeWidth();
		graphics.setStroke(new BasicStroke(st));
		graphics.setPaint(getModelElement().getStroke());
		graphics.draw(getShape());
	}

	@Override
	protected void createElement(Element elem) {
		super.createElement(elem);
		elem.setAttribute("fill", colorToString(getModelElement().getFill()));
	}
	
	/**
	 * @return the component
	 */
	public GraphicsComponent getComponent() {
		return component;
	}

}
