package ru.amse.fedorov.plainsvg.presentation.markers;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;

import ru.amse.fedorov.plainsvg.geom.GeomUtils;
import ru.amse.fedorov.plainsvg.gui.ComponentSelectedList;
import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.presentation.elements.ElementPresentation;
import ru.amse.fedorov.plainsvg.presentation.elements.ShapePresentation;

public abstract class RotateMarker<E extends ShapePresentation<?>> extends
		Marker<E> {
	
	protected final int DIST = 20;

	private final ComponentSelectedList selectedList;

	private final boolean inside;

	private final GraphicsComponent component;

	public RotateMarker(E pres, GraphicsComponent comp, Cursor cursor, boolean f) {
		super(pres, Color.GREEN, cursor);
		component = comp;
		selectedList = component.getSelectedList();
		inside = f;
	}


	@Override
	public void movedUtransformed(Point2D p) {
		double angle;
		Point2D center = getPresentation().getModelElement().getCenter();
		center = getPresentation().getTransform().transform(center, null);
		p = getPresentation().getTransform().transform(p, null);
		Point2D transformed = getPresentation().getTransform().transform(
				getPoint(), null);
		angle = GeomUtils.angle(transformed, center, p);
		for (ElementPresentation<?> elem : selectedList) {
			Point2D myCenter = elem.getModelElement().getCenter();
			myCenter = elem.getTransform().transform(myCenter, null);
			AffineTransform at = new AffineTransform();
			at.rotate(angle, myCenter.getX(), myCenter.getY());
			at.concatenate(elem.getTransform());
			elem.setTransform(at);
		}
	}

	/**
	 * @return the component
	 */
	public GraphicsComponent getComponent() {
		return component;
	}


	/**
	 * @return the secondBigger
	 */
	protected boolean isInside() {
		return inside;
	}

}
