package ru.amse.fedorov.plainsvg.tools;

import java.awt.geom.Point2D;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.model.elements.SVGElement;
import ru.amse.fedorov.plainsvg.presentation.elements.StrokedPresentation;

public abstract class CreateDiagonaledTool<E extends SVGElement, P extends StrokedPresentation<?>>
		extends CreateTool<E, P> {

	CreateDiagonaledTool(GraphicsComponent component) {
		super(component);
	}

	@Override
	public final void mouseDragged(Point2D point, boolean isControlDown) {
		StrokedPresentation<?> stroked = getPresentation();
		if (isControlDown) {
			double dx = point.getX() - getPointWherePressed().getX();
			double dy = point.getY() - getPointWherePressed().getY();
			double max = Math.max(Math.abs(dx), Math.abs(dy));
			double px = getPointWherePressed().getX() + max * Math.signum(dx);
			double py = getPointWherePressed().getY() + max * Math.signum(dy);
			Point2D p = new Point2D.Double(px, py);
			stroked.setDiagonal(p, getPointWherePressed());
		} else {
			stroked.setDiagonal(point, getPointWherePressed());
		}
	}

	@Override
	public final void mouseReleased(Point2D point, boolean isControlDown) {
		if (isElementSingular()) {
			getElementList().remove(getPresentation());
		} else {
			getPresentation().addMarkers();
			getSelectedList().add(getPresentation());
		}
		getComponent().setDefaultCurrentTool();
	}

	protected abstract void createElement(Point2D point);

	@Override
	public final void mousePressed(Point2D point, boolean isControlDown) {
		getSelectedList().clear();
		getPointWherePressed().setLocation(point);
		createElement(point);
		getElement().addElementChangedListener(
				getComponent().getSuperListener());
		getElementList().addToEnd(getPresentation());
	}

}
