package ru.amse.fedorov.plainsvg.tools;

import java.awt.geom.Point2D;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.model.elements.SVGEllipse;
import ru.amse.fedorov.plainsvg.presentation.elements.EllipsePresentation;

/**
 * Tool for creating ellipses.
 * 
 * @author Alexey Fedorov
 */
public class CreateEllipseTool extends CreateDiagonaledTool<SVGEllipse, EllipsePresentation> {

	/*
	 * Mediator for created ellipse.
	 */
	private EllipsePresentation ellipsePresentation;

	/*
	 * Created svg-ellipse
	 */
	private SVGEllipse svgEllipse;

	/**
	 * Creates instance of tool "Create ellipse".
	 * 
	 * @param component
	 *            graphics component to draw the ellipse
	 */
	public CreateEllipseTool(GraphicsComponent component) {
		super(component);
	}

	@Override
	protected void createElement(Point2D point) {
		svgEllipse = new SVGEllipse(getPointWherePressed().getX(),
				getPointWherePressed().getY());
		ellipsePresentation = new EllipsePresentation(svgEllipse,
				getComponent());
	}

	@Override
	protected EllipsePresentation getPresentation() {
		return ellipsePresentation;
	}

	@Override
	protected SVGEllipse getElement() {
		return svgEllipse;
	}

	@Override
	protected boolean isElementSingular() {
		return (Math.abs(svgEllipse.getRadiusX()) < EPS)
				&& (Math.abs(svgEllipse.getRadiusY()) < EPS);
	}

}
